# -*- coding: utf-8 -*-
#
# Copyright 2021-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

"""
Hook which moves the user to transfer_school when only one school
is assigned to it. If the user is assigned to multiple schools, no delete
action is triggered. Removing from the primary school is not handled by this
hook but handled by ucs by modifing the users school.
"""

from builtins import str
import types
import json

from ucsschool.lib.models.hook import Hook
from ucsschool.lib.models.user import User
from ucsschool.importer.models.import_user import ImportUser
from univention.udm.exceptions import ModifyError


class DeactivateUsersBeforeDeletion(Hook):
	model = User  # hook will only run for objects of this type
	priority = {
		"pre_create": None,
		"post_create": None,
		"pre_modify": None,
		"post_modify": None,
		"pre_move": None,
		"post_move": None,
		"pre_remove": 200,  # hook will run before a remove operation
		"post_remove": None,
	}

	def __init__(self, lo, *args, **kwargs):
		super(DeactivateUsersBeforeDeletion, self).__init__(lo, *args, **kwargs)

	@property
	def class_name(self):
		return type(self).__name__

	def info(self, message):
		self.logger.info("%s: %s", self.class_name, message)

	def warn(self, message):
		self.logger.warn("%s: %s", self.class_name, message)

	def error(self, message):
		self.logger.error("%s: %s", self.class_name, message)

	def pre_remove(self, obj):  # type: (User) -> None
		"""
		Move user to transfer_school when only one school is assigned
		and prevent the user object from being deleted.
		:param User obj: the User instance, that was just modified in LDAP
		:return: None
		"""
		# Skip this hook when not called by the schoolwizards/user module
		if isinstance(obj, ImportUser):
			self.info("Skipping hook.")
			return

		def get_limbo_ou():  # type: () -> str
			# get limbo_ou from SiSoSpi-import-json
			importConfig = "/var/lib/ucs-school-import/configs/user_import.json"
			try:
				with open(importConfig, "r") as ic:
					jic = json.load(ic)
			except OSError as exc:
				self.warn("%r: Can't open file %r" % (exc, importConfig))

			if "limbo_ou" in jic and jic["limbo_ou"] is not None:
				limbo_ou = str(jic["limbo_ou"])
			else:
				limbo_ou = "DeaktivierteKonten"

			return limbo_ou

		def overwrite_remove(self, lo):
			"""
			Overwrite the original remove_without_hooks function but
			without the remove instructions.
			Otherwise the post_remove hook will be overwritten or the dn
			of the object is None!
			"""
			udm_obj = obj.get_udm_object(lo)
			if not udm_obj:
				return False
			return True

		def change_school(new_school):
			result = obj.change_school(new_school, self.lo)
			if result:
				self.info("Moved user %r to school %r." % (obj.name, new_school))
			else:
				self.info("Can't move %r to school %r. Aborting." % (obj.name, new_school))

		transfer_school = get_limbo_ou()

		"""
		If the user is assigned to only one school remove the
		user from primary school and move to transfer_school.
		"""

		# Partly work with the udm object to trick the UCS@School lib
		udm_obj = obj.get_udm_object(self.lo)

		if len(udm_obj["school"]) == 1:
			# If the user already resides in limbo, allow them to finally die
			if obj.school == transfer_school:
				return
			# Prevent deletion of user object
			# obj.remove_without_hooks = types.MethodType(overwrite_remove, self.model)
			obj.remove_without_hooks = types.MethodType(overwrite_remove, self.lo)

			obj.disabled = "1"
			self.info("Disabled user %r." % obj.name)

			self.info("Instead of deleting move user %r to school %r." % (obj.name, transfer_school))
			old_school = obj.get_school_obj(self.lo).name
			# Move user from old_school to transfer_school
			change_school(transfer_school)
			self.info("Remove user %r from school groups." % obj.name)
			obj.remove_from_groups_of_school(old_school, self.lo)

			# refresh object because the dn changed
			udm_obj = obj.get_udm_object(self.lo)
			try:
				udm_obj.modify()
			except ModifyError as exc:
				self.error("%r: Cannot modify %r." % exc, obj.name)

		"""
		If the user is assigned to multiple schools no delete action is triggered.
		Removing from the primary school is not handled by this hook but by ucs by
		modifing the users school.
		"""

		self.info("Done!")
