# -*- coding: utf-8 -*-
#
# Copyright 2021-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

"""
Hook which appends the old mailPrimaryAddress to mailAlternativeAddress
and current datetime to mailAddressExpired if firstname or lastname gets changed.
Based on the date, later the mailalias can be removed after the time has expired.
"""

from datetime import datetime
from ucsschool.lib.models.hook import Hook
from ucsschool.lib.models.user import User
from univention.udm.exceptions import ModifyError


class GenerateMailAlias(Hook):
	model = User  # hook will only run for objects of this type
	priority = {
		"pre_create": None,
		"post_create": None,
		"pre_modify": 990,  # hook will run before a modify operation
		"post_modify": 990,  # hook will run after a modify operation
		"pre_move": None,
		"post_move": None,
		"pre_remove": None,
		"post_remove": None,
	}

	@property
	def class_name(self):
		return type(self).__name__

	def info(self, message):
		self.logger.info("%s: %s", self.class_name, message)

	def error(self, message):
		self.logger.error("%s: %s", self.class_name, message)

	def pre_modify(self, obj):  # type: (SchoolUser) -> None
		"""
		To get the unchanged user data to compare them later with
		the changed user data in post_modify hook.
		"""
		self.info("User %r will be modified." % obj.name)
		udm_obj = obj.get_udm_object(self.lo)
		self.old_lastname = udm_obj["lastname"]
		self.old_firstname = udm_obj["firstname"]
		self.old_email = udm_obj["mailPrimaryAddress"]
		self.info("Done!")

	def post_modify(self, obj):  # type: (SchoolUser) -> None
		"""
		Add current mailPrimaryAddress to mailAlternativeAddress and current datetime
		to mailAddressExpired if family or given name changes.
		Based on the date, later the mailalias can be removed after the time has expired.
		:param SchoolUser obj: the User instance, that was just modified in LDAP
		:return: None
		"""
		self.info("User %r was modified." % obj.name)

		# Get the old UDM object
		udm_obj = obj.get_udm_object(self.lo)

		# react on changes of firstname or lastname
		if (self.old_firstname != udm_obj["firstname"] or self.old_lastname != udm_obj["lastname"]):
			self.info("User attributes (firstname, lastname) of user %r was changed." % obj.name)
			if self.old_email:
				self.info("Add current mailPrimaryAddress %r of user %r to mailAlternativeAddress." % (self.old_email, obj.name))
				udm_obj["mailAlternativeAddress"].append(self.old_email)

				"""
				The UDM syntax DateTimeTimezone parses this internally as
				list elems: [date, time, timezone], with all fields optional
				example: "2022-02-02 12:30 UTC" becomes ["2022_02-02", "12:30", "UTC"]
				"""
				udm_obj["mailAddressExpired"].append(datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
				self.info("mailAddressExpired changed %s" % udm_obj["mailAddressExpired"])
				try:
					udm_obj.modify()
				except ModifyError as exc:
					self.error("%r: Cannot modify mailAlternativeAddress or mailAddressExpired of user %r." % (exc, obj.name))
			else:
				self.error("User %r has no e-mail. Skipping.." % obj.name)
		else:
			self.info("No changes of user attributes (firstname, lastname) of user %r detected." % obj.name)

		self.info("Done!")
