# -*- coding: utf-8 -*-
#
# Copyright 2021-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

"""
Hook which generates a new unique username if firstname or lastname
gets changed.
"""

from ucsschool.importer.configuration import Configuration, setup_configuration
from ucsschool.importer.exceptions import InitialisationError
from ucsschool.importer.factory import setup_factory
from ucsschool.importer.frontend.user_import_cmdline import (
	UserImportCommandLine,
)
from ucsschool.importer.models.import_user import ImportUser
from ucsschool.lib.models.hook import Hook
from ucsschool.lib.models.user import (
	Staff,
	Student,
	Teacher,
	TeachersAndStaff,
	SchoolAdmin,
	User,
)

import os.path


class GenerateUserName(Hook):
	model = User  # hook will only run for objects of this type
	priority = {
		"pre_create": None,
		"post_create": None,
		"pre_modify": 1000,  # hook will run before the UDM object is modified
		"post_modify": None,
		"pre_move": None,
		"post_move": None,
		"pre_remove": None,
		"post_remove": None,
	}

	def __init__(self, *args, **kwargs):
		super(GenerateUserName, self).__init__(*args, **kwargs)
		# Important: only initialize the config if it's not already loaded, needed for the schoolwizard
		# If we are in the importer scope, the config is already initialized (as a singleton).
		try:
			self.config = Configuration()
		except InitialisationError:
			self.config = self.get_import_configuration()
		self.factory = setup_factory(self.config["factory"])

	def get_import_configuration(self):  # type: () -> config
		"""
		Get the Configuration from the UCS@School HTTP API Importer
		to properly initialize the import factory.
		:return: config
		"""
		ui = UserImportCommandLine()
		config_files = ui.configuration_files
		for fn in [
			"/usr/share/ucs-school-import/configs/user_import_http-api.json",
			"/var/lib/ucs-school-import/configsuser_import_http-api.json"
		]:
			if os.path.exists(fn):
				config_files.append(fn)
		# We use DEMOSCHOOL as dummy school  and student as dummy role here
		# to convince the importer to have a proper config.
		# It's not needed during this hook as we are only interested in the username
		config = setup_configuration(config_files, school="DEMOSCHOOL", user_role="student")
		ui.setup_logging(config["verbose"], config["logfile"])
		return config

	@property
	def class_name(self):
		return type(self).__name__

	def info(self, message):
		self.logger.info("%s: %s", self.class_name, message)

	def error(self, message):
		self.logger.error("%s: %s", self.class_name, message)

	def pre_modify(self, obj):  # type: (SchoolUser) -> None
		"""
		Generate new unique username if family or given name changes.
		:param SchoolUser obj: the User instance, that was just modified in LDAP
		:return: None
		"""
		# Skip this hook in the importer, as the importer already knows how to generate
		# usernames and throws a traceback when run twice
		if isinstance(obj, ImportUser):
			self.info("Skipping hook.")
			return
		self.info("User %r was modified." % obj.name)

		# Get the old UDM object
		udm_obj = obj.get_udm_object(self.lo)
		# Get dict of underlying unmodified UDM object
		oldinfo = udm_obj.info

		# react on changes of firstname or lastname
		if (oldinfo["firstname"] != obj.firstname or oldinfo["lastname"] != obj.lastname):
			self.info("User attributes (firstname, lastname) of user %r was changed." % obj.name)
			userName = self.generate_username(obj)
			if userName:
				self.info("Change username %r to %r." % (obj.name, userName))
				obj.name = userName
			else:
				self.error("No valid user model of user %r found." % obj.name)
		else:
			self.info("No changes of user attributes (firstname, lastname) of user %r detected." % obj.name)

		self.info("Done!")

	def generate_username(self, obj):  # type: (SchoolUser) -> str
		"""
		Call make_username() from the UCS@School import_user class
		:param SchoolUser obj: UCS@School user object
		:return: formatted string
		:rtype: str
		"""
		# Look for the right type to use the correct role.
		# One of `staff`, `student`, `teacher`, `teacher_and_staff`
		# make_import_user takes a list of user roles
		if isinstance(obj, Student):
			user = self.factory.make_import_user(["student"])
		elif isinstance(obj, TeachersAndStaff):
			user = self.factory.make_import_user(["teacher_and_staff"])
		elif isinstance(obj, Teacher):
			user = self.factory.make_import_user(["teacher"])
		elif isinstance(obj, Staff):
			user = self.factory.make_import_user(["staff"])
		elif isinstance(obj, SchoolAdmin):
			# the importer does not know plain schooladmins yet, so we get a base user back
			user = self.factory.make_import_user([])
		else:
			return

		user.name = ""
		user.firstname = obj.firstname
		user.lastname = obj.lastname

		return user.make_username()
