# -*- coding: utf-8 -*-
#
# Copyright 2021-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

"""
Hook which sets the ucsschoolPurgeTimestamp on user deactivation.
If the user gets (re-) activated, then the purgeTimeStamp is deleted
"""

from datetime import datetime, timedelta

from ucsschool.lib.models.hook import Hook
from ucsschool.lib.models.user import User
from univention.udm.exceptions import ModifyError


class PurgeDeactivatedUsers(Hook):
	model = User  # hook will only run for objects of this type
	priority = {
		"pre_create": None,
		"post_create": None,
		"pre_modify": None,
		"post_modify": 100,  # hook will run after a modify operation
		"pre_move": None,
		"post_move": None,
		"pre_remove": None,
		"post_remove": None,
	}

	@property
	def class_name(self):
		return type(self).__name__

	def info(self, message):
		self.logger.info("%s: %s", self.class_name, message)

	def error(self, message):
		self.logger.error("%s: %s", self.class_name, message)

	def post_modify(self, obj):  # type: (SchoolUser) -> None
		"""
		Set actual date + interval from UCRV as ucsschoolPurgeTimestamp
		:param SchoolUser obj: the User instance, that was just modified in LDAP
		:return: None
		"""
		self.info("User %r was modified." % obj.name)
		udm_obj = obj.get_udm_object(self.lo)

		if udm_obj["disabled"] == '0':
			self.info("User %r is activated." % obj.name)
			if udm_obj.get('ucsschoolPurgeTimestamp'):
				udm_obj["ucsschoolPurgeTimestamp"] = None
				try:
					udm_obj.modify()
				except ModifyError as exc:
					self.error("%r: Cannot remove ucsschoolPurgeTimestamp from %r." % (exc, obj.name))
			else:
				self.info("User %r has no purgeTimeStamp set, Skipping..." % obj.name)
		else:
			self.info("User %r is deactivated." % obj.name)
			# Get UCRV dataport/schuposh/ucl/purge_period, use 84 if empty
			expiryVar = int(self.ucr['dataport/schuposh/ulc/purge_period'])
			if expiryVar is None:
				expiryVar = 84

			# the ucsschoolPurgeTimestamp follows the syntax YYYY-MM-DD
			purgeTimeStamp = (datetime.now() + timedelta(days=expiryVar)).strftime("%Y-%m-%d")

			self.info("Setting purgeTimeStamp to %r for user %r..." % (purgeTimeStamp, obj.name))
			udm_obj["ucsschoolPurgeTimestamp"] = purgeTimeStamp
			try:
				udm_obj.modify()
			except ModifyError as exc:
				self.error("%r: Cannot set ucsschoolPurgeTimestamp for %r." % (exc, obj.name))

		self.info("Done!")
