# -*- coding: utf-8 -*-
#
# Copyright 2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

"""
Hook which removes leftover admin groups from school admins after school change.
Workaround for https://forge.univention.org/bugzilla/show_bug.cgi?id=54368
"""

from ucsschool.lib.models.hook import Hook
from ucsschool.lib.models.user import User, SchoolAdmin
from univention.udm.exceptions import ModifyError


class RemoveOldSchoolAdminGroups(Hook):
	model = User  # hook will only run for objects of this type
	priority = {
		"pre_create": None,
		"post_create": None,
		"pre_modify": None,
		"post_modify": 200,   # hook is called after a modification
		"pre_move": None,
		"post_move": None,
		"pre_remove": None,
		"post_remove": None,
	}

	def __init__(self, lo, *args, **kwargs):
		super(RemoveOldSchoolAdminGroups, self).__init__(lo, *args, **kwargs)

	@property
	def class_name(self):
		return type(self).__name__

	def info(self, message):
		self.logger.info("%s: %s", self.class_name, message)

	def warn(self, message):
		self.logger.warn("%s: %s", self.class_name, message)

	def error(self, message):
		self.logger.error("%s: %s", self.class_name, message)

	def post_modify(self, obj):    # type: (SchoolUser) -> None

		# Skip this hook when not called by the schoolwizards/user module
		if not isinstance(obj, SchoolAdmin):
			self.info("No school admin, skipping hook.")
			return

		udm_obj = obj.get_udm_object(self.lo)

		self.info("Workaround: Remove additional groups from SchoolAdmin...")
		udm_obj["groups"] = obj.groups_used(self.lo)
		try:
			udm_obj.modify()
		except ModifyError as exc:
			self.error("%r: Cannot remove groups from %r." % exc, obj.name)
