# -*- coding: utf-8 -*-
#
# Copyright 2021-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

"""
Hook which checks the user data for modification.
And in case of change sends an email to the appropriate schools of the user.
"""

import csv
import os
from mailer import Mailer, Message
from ucsschool.importer.models.import_user import ImportUser
from ucsschool.lib.models.hook import Hook
from ucsschool.lib.models.user import User

CSV_SCHOOL_TO_EMAIL_COLUMN_OU = "Dienststellennummer"
CSV_SCHOOL_TO_EMAIL_COLUMN_EMAIL = "E-Mail"
CSV_SCHOOL_TO_EMAIL_DEFAULT_FILE = "/var/lib/ucs-school-import/configs/school-to-email.csv"

MAIL_SENDER_DEFAULT = "noreply@schule-sh.de"
MAIL_SUBJECT = "Stammdatenänderung im Schulportal"
MAIL_BODY = u"""Sehr geehrte Damen und Herren,

die Stammdaten zu einer Identität Ihrer Schule im Schulportal haben sich wie folgt geändert:

Vorname: <firstname>

Nachname: <lastname>

Benutzername: <username>

E-Mail-Adresse: <mail_address>

Personalnummer: <record_uid>

Das Passwort wurde nicht verändert und kann wie zuvor verwendet werden.

Bitte teilen Sie der betroffenen Person die Stammdatenänderung mit. Bitte schicken Sie die Information nicht an die dienstliche E-Mail-Adresse (…@schule-sh.de) der betroffenen Person, da der neue Benutzername für die Anmeldung im Schulportal (und somit auch den Zugriff auf das E-Mail-Postfach) benötigt wird.

Bei Rückfragen nutzen Sie gern unseren Supportkanal: IQSH-Helpdesk (https://www.secure-lernnetz.de/helpdesk/).

Dies ist eine systemseitig erzeugte E-Mail. Bitte antworten Sie nicht darauf.

Mit freundlichen Grüßen

Ihr Schulportal-Team
"""


class SendEmailUsernameChanged(Hook):
	model = User  # hook will only run for objects of this type
	priority = {
		"pre_create": None,
		"post_create": None,
		"pre_modify": 800,
		"post_modify": 800,  # hook will run after a modify operation
		"pre_move": None,
		"post_move": None,
		"pre_remove": None,
		"post_remove": None,
	}

	def __init__(self, lo, *args, **kwargs):
		super(SendEmailUsernameChanged, self).__init__(lo, *args, **kwargs)
		self.old_firstname = None
		self.old_lastname = None

	@property
	def class_name(self):
		return type(self).__name__

	def info(self, message):
		self.logger.info("%s: %s", self.class_name, message)

	def warn(self, message):
		self.logger.warn("%s: %s", self.class_name, message)

	def error(self, message):
		self.logger.error("%s: %s", self.class_name, message)

	def pre_modify(self, obj):  # type: (SchoolUser) -> None
		"""
		To get the unchanged user data to compare them later with
		the changed user data in post_modify hook.
		"""
		self.info("User %r will be modified." % obj.name)
		udm_obj = obj.get_udm_object(self.lo)
		self.old_lastname = udm_obj["lastname"]
		self.old_firstname = udm_obj["firstname"]
		self.old_email = udm_obj["mailPrimaryAddress"]
		self.info("Done!")

	def post_modify(self, obj):  # type: (User) -> None
		"""
		Comparison of old and new user data. If there are changes
		in firstname or lastname, an email is sent to the user's school(s).
		"""

		# Skip this hook in the importer, as the importer already knows how to generate
		# usernames and throws a traceback when run twice
		if isinstance(obj, ImportUser):
			self.info("Skipping hook.")
			return

		self.info("User %r was modified." % obj.name)
		udm_obj = obj.get_udm_object(self.lo)

		if (self.old_firstname != udm_obj["firstname"] or self.old_lastname != udm_obj["lastname"]):
			self.info("User attributes (firstname, lastname) of user %r was changed." % obj.name)
			# Send emails to school with new user attributes.
			self.mail_to_schools({
				"firstname": udm_obj["firstname"],
				"lastname": udm_obj["lastname"],
				"username": udm_obj["username"],
				"mail_address": udm_obj["mailPrimaryAddress"],
				"record_uid": udm_obj["ucsschoolRecordUID"]
			}, udm_obj["school"])
		else:
			self.info("No changes of user attributes (firstname, lastname) of user %r detected." % obj.name)

		self.info("Done!")

	def mail_to_schools(self, user_attr, schools):  # type: (dict, list) -> None
		"""
		Send an email to each school with the user's changes.
		:param user_attr: User data / changes which is sent with the e-mail.
			{
				'firstname': <firstname>,
				'lastname': <lastname>,
				'username': <username>,
				'mail_address': <mail_addr>,
				'record_uid': <record_uid>
			}
		:param schools: List of school OUs.
		"""
		self.info("Sending email(s) to school(s): %r." % (' '.join(schools)))
		to_addresses = []
		ou2email = self.get_ou_email_mapping()
		if not ou2email:
			return
		for school in schools:
			if school.lower() in ou2email:
				to_addresses.append(ou2email[school.lower()])
		if not to_addresses:
			self.info("No matching schools found. No e-mails are sent.")
			return
		sender = Mailer("localhost")
		message = Message(
			From=MAIL_SENDER_DEFAULT,
			To=to_addresses,
			Subject=MAIL_SUBJECT,
			charset="utf-8",
		)
		message.Body = self.replace_mail_user_attributes(user_attr)
		sender.send(message)
		self.info("OK: Sent email(s) to school(s).")

	def replace_mail_user_attributes(self, user_attr):  # type: (dict) -> str
		"""
		Replace tags (<.*>) from user_attr (keys) in MAIL_BODY
		with user attributes (values).

		:param user_attr: User data which will be inserted into the body of the email.
			{
				'firstname': <firstname>,
				'lastname': <lastname>,
				'username': <username>,
				'mail_address': <mail_addr>,
				'record_uid': <record_uid>
			}
		"""
		msg_body = MAIL_BODY
		for key, value in list(user_attr.items()):
			msg_body = msg_body.replace("<%s>" % key, value)

		return msg_body

	def get_ou_email_mapping(self, csv_file=CSV_SCHOOL_TO_EMAIL_DEFAULT_FILE):  # type: (str) -> dict
		"""
		Converts a CSV file with the following format:
			Dienststellennummer,E-Mail
			DEMOSCHOOL1,mail1@example.org
			DEMOSCHOOL2,mail2@example.org
		into a Dict:
			{
				'DEMOSCHOOL1': 'mail1@example.org',
				'DEMOSCHOOL2': 'mail2@example.org'
			}
		"""
		ou2email = {}
		if not os.path.isfile(csv_file):
			self.warn("File %r does not exists. Abort!" % csv_file)
			return
		with open(csv_file) as csvfile:
			dialect = csv.Sniffer().sniff(csvfile.read(1024))
			csvfile.seek(0)
			reader = csv.DictReader(csvfile, dialect=dialect)
			for row in reader:
				if not row:
					raise ValueError("CSV file is empty.")
				for column in (CSV_SCHOOL_TO_EMAIL_COLUMN_OU, CSV_SCHOOL_TO_EMAIL_COLUMN_EMAIL):
					if column not in row:
						raise ValueError(
							"Missing column {!r} in {!r}.".format(
								column, csv_file)
						)
					ou2email[row[CSV_SCHOOL_TO_EMAIL_COLUMN_OU].lower()] = row[
						CSV_SCHOOL_TO_EMAIL_COLUMN_EMAIL
					].lower()

		return ou2email
